from typing import TypeVar, Callable
from reactivex import Observable, operators

_T = TypeVar('_T')

def item_at_index(index: int) -> Callable[[Observable[list[_T]]], Observable[_T]]:
    return operators.map(
        lambda x: x[index]
    )


from dataclasses import dataclass
from decimal import Decimal
import decimal

from logging import getLogger

logger = getLogger(__name__)

def compute_grid_prices( * ,
    price_decimal_places: int,
    center_price: Decimal,
    gap: Decimal,
    order_count: int,
) -> list[Decimal]:
    with decimal.localcontext() as context:
        context.rounding = decimal.ROUND_DOWN
        initial_price = round(center_price - gap, price_decimal_places)
    prices = []
    for i in range(order_count):
        is_even = bool(i % 2)
        multiplier = (-1 * (i // 2)) if is_even else (2 + i // 2)
        prices.append(initial_price + gap * multiplier)
    return prices


__all__ = [
    "compute_grid_prices"
]


from decimal import Decimal

def price_volume(x: tuple[float, float]):
    return Decimal(x[0]), Decimal(x[1])

def get_price_from_tuple(x: tuple[float, float]) -> Decimal:
    return Decimal(x[0])

def get_volume_from_tuple(x: tuple[float, float]) -> Decimal:
    return Decimal(x[1])

def get_bids(x: dict) -> list[tuple[float, float]]:
    return x['bids']

def get_asks(x: dict) -> list[tuple[float, float]]:
    return x['asks']

def top_bid_price(x: dict) -> Decimal:
    return get_price_from_tuple(get_bids(x)[0])

def lowest_ask_price(x: dict) -> Decimal:
    return get_price_from_tuple(get_asks(x)[0])


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['elm_framework_helpers',
 'elm_framework_helpers.ccxt',
 'elm_framework_helpers.ccxt.orderbook',
 'elm_framework_helpers.operators',
 'elm_framework_helpers.output',
 'elm_framework_helpers.strategies.grid']

package_data = \
{'': ['*']}

install_requires = \
['reactivex>=4.0.4,<5.0.0']

setup_kwargs = {
    'name': 'elm-framework-helpers',
    'version': '0.1.5',
    'description': '',
    'long_description': "# Helpers for the ELM framework scripts\n\n## Logging\n\nHere's a sample logging for production:\n\n```\n[loggers]\nkeys=root,Rx,rawsocket\n\n[handlers]\nkeys=consoleHandler,fileHandler,socketFileHandler\n\n[formatters]\nkeys=fileFormatter,consoleFormatter,socketFileFormatter\n\n[logger_root]\nlevel=DEBUG\nhandlers=consoleHandler,fileHandler\n\n[logger_Rx]\nlevel=INFO\nhandlers=consoleHandler\nqualname=Rx\n\n[logger_rawsocket]\nqualname=bittrade_kraken_websocket.connection.generic.raw\nhandlers=socketFileHandler\n\n[handler_consoleHandler]\nclass=StreamHandler\nlevel=INFO\nformatter=consoleFormatter\nargs=(sys.stdout,)\n\n[handler_fileHandler]\nclass=FileHandler\nlevel=DEBUG\nformatter=fileFormatter\nargs=('logfile.log',)\n\n[handler_socketFileHandler]\nclass=FileHandler\nlevel=DEBUG\nformatter=socketFileFormatter\nargs=('raw_socket.log',)\n\n[formatter_fileFormatter]\nformat=%(asctime)s   - %(name)s - %(levelname)s - %(message)s\ndatefmt=%H:%M:%S\n\n[formatter_socketFileFormatter]\nformat=%(asctime)s - %(message)s\ndatefmt=%H:%M:%S\n\n[formatter_consoleFormatter]\nformat=%(asctime)s   - %(levelname)s - %(message)s\ndatefmt=%H:%M:%S\n```",
    'author': 'Mat',
    'author_email': 'mathieu@redapesolutions.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/TechSpaceAsia/elm-framework-helpers',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from .orderbook.extract import get_asks, get_bids, get_price_from_tuple, get_volume_from_tuple, price_volume

__all__ = [
    "get_asks", "get_bids", "get_price_from_tuple", "get_volume_from_tuple", "price_volume",
]